@@ -4,14 +4,14 @@
 #' @template ncbi
 #' @param gene (character) Gene or genes (in a vector) to search for.
 #' See examples.
-#' @param ... Curl options passed on to \code{\link[httr]{GET}}
+#' @param ... Curl options passed on to [crul::verb-GET]
 #' @details Removes predicted sequences so you don't have to remove them.
-#'   	Predicted sequences are those with accession numbers that have "XM_" or
-#' 		"XR_" prefixes. This function retrieves one sequences for each species,
-#'   	picking the longest available for the given gene.
-#' @return Data.frame of results.
-#' @seealso \code{\link{ncbi_searcher}}, \code{\link{ncbi_byid}}
-#' @author Scott Chamberlain \email{myrmecocystus@@gmail.com}
+#' Predicted sequences are those with accession numbers that have "XM_" or
+#' "XR_" prefixes. This function retrieves one sequences for each species,
+#' picking the longest available for the given gene.
+#' @return data.frame
+#' @seealso [ncbi_searcher()], [ncbi_byid()]
+#' @author Scott Chamberlain
 #' @examples \dontrun{
 #' # A single species
 #' ncbi_byname(taxa="Acipenser brevirostrum")
@@ -37,14 +37,18 @@ ncbi_byname <- function(taxa, gene="COI", seqrange="1:3000", getrelated=FALSE,
       db = "nuccore",
       term = paste(xx, "[Organism] AND", genes_, "AND",
                    seqrange, "[SLEN]", collapse = " "),
-      RetMax = 500)
+      RetMax = 500,
+      api_key = ncbi_key())
 
+    con <- crul::HttpClient$new("https://eutils.ncbi.nlm.nih.gov",
+      opts = list(...))
+    w <- con$get("entrez/eutils/esearch.fcgi", query = query)
+    w$raise_for_status()
     out <-
-      xml2::xml_find_all(xml2::read_xml(content(GET("https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi",
-                             query = query), "text", encoding = "UTF-8")), "//eSearchResult")[[1]]
+      xml2::xml_find_all(xml2::read_xml(w$parse("UTF-8")), "//eSearchResult")[[1]]
     if (as.numeric(xml2::xml_text(xml2::xml_find_all(out, "//Count")[[1]])) == 0) {
       message(paste("no sequences of ", gene, " for ", xx, " - getting other sp.", sep = ""))
-      if (getrelated == FALSE) {
+      if (!getrelated) {
         mssg(verbose, paste("no sequences of ", gene, " for ", xx, sep = ""))
         res <- data.frame(
           xx, NA_character_, NA_real_, NA_character_, NA_real_, NA_character_, NA_character_,
@@ -53,10 +57,14 @@ ncbi_byname <- function(taxa, gene="COI", seqrange="1:3000", getrelated=FALSE,
       } else {
         mssg(verbose, "...retrieving sequence IDs for related species...")
         newname <- strsplit(xx, " ")[[1]][[1]]
-        query <- list(db = "nuccore", term = paste(newname, "[Organism] AND", genes_, "AND", seqrange, "[SLEN]", collapse = " "), RetMax = 500)
+        query <- list(db = "nuccore",
+          term = paste(newname, "[Organism] AND", genes_, "AND", seqrange, "[SLEN]", collapse = " "), 
+          RetMax = 500,
+          api_key = ncbi_key())
+        bb <- con$get("entrez/eutils/esearch.fcgi", query = query)
+        bb$raise_for_status()
         out <-
-          xml2::xml_find_all(xml2::read_xml(content(GET("https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi", query = query),
-                                                    "text", encoding = "UTF-8")), "//eSearchResult")[[1]]
+          xml2::xml_find_all(xml2::read_xml(bb$parse("UTF-8")), "//eSearchResult")[[1]]
         if (as.numeric(xml2::xml_text(xml2::xml_find_all(out, "//Count")[[1]])) == 0) {
           mssg(verbose, paste("no sequences of ", gene, " for ", xx, " or ", newname, sep = ""))
           res <- data.frame(
@@ -66,20 +74,25 @@ ncbi_byname <- function(taxa, gene="COI", seqrange="1:3000", getrelated=FALSE,
         } else {
           ## For each species = get GI number with longest sequence
           mssg(verbose, "...retrieving sequence ID with longest sequence length...")
-          querysum <- list(db = "nucleotide", id = paste(make_ids(out), collapse = " ")) # construct query for species
+          querysum <- list(db = "nucleotide",
+            id = paste(make_ids(out), collapse = " "),
+            api_key = ncbi_key())
+          z <- con$get("entrez/eutils/esummary.fcgi", query = querysum)
+          z$raise_for_status()
           res <- parse_ncbi(xx,
                 xml2::xml_find_all(
-                  xml2::read_xml(content(GET("https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi",
-                                   query = querysum), "text", encoding = "UTF-8")), "//eSummaryResult"), verbose)
+                  xml2::read_xml(z$parse("UTF-8")), "//eSummaryResult"), verbose)
         }
       }
     } else {
       ## For each species = get GI number with longest sequence
       mssg(verbose, "...retrieving sequence ID with longest sequence length...")
-      querysum <- list(db = "nucleotide", id = paste(make_ids(out), collapse = " ")) # construct query for species
-      res <- parse_ncbi(xx, xml2::xml_find_all(xml2::read_xml(content( # API call
-        GET("https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi",
-            query = querysum), "text", encoding = "UTF-8")), "//eSummaryResult")[[1]], verbose)
+      # construct query for species
+      querysum <- list(db = "nucleotide", api_key = ncbi_key(),
+        id = paste(make_ids(out), collapse = " "))
+      z <- con$get("entrez/eutils/esummary.fcgi", query = querysum)
+      txt <- z$parse("UTF-8")
+      res <- parse_ncbi(xx, xml2::xml_find_all(xml2::read_xml(txt), "//eSummaryResult")[[1]], verbose)
     }
 
     mssg(verbose, "...done.")
@@ -90,7 +103,7 @@ ncbi_byname <- function(taxa, gene="COI", seqrange="1:3000", getrelated=FALSE,
   if (length(taxa) == 1){ foo_safe(taxa) } else { lapply(taxa, foo_safe) }
 }
 
-parse_ncbi <- function(xx, z, verbose){
+parse_ncbi <- function(xx, z, verbose) {
   names <- xml2::xml_attr(xml2::xml_find_all(z, "//Item"), "Name") # gets names of values in summary
   prd <- xml2::xml_text(xml2::xml_find_all(z, '//Item[@Name="Caption"]')) #  get access numbers
   prd <- sapply(prd, function(x) strsplit(x, "_")[[1]][[1]], USE.NAMES=FALSE)
@@ -105,8 +118,12 @@ parse_ncbi <- function(xx, z, verbose){
   }
   ## Get sequence from previous
   mssg(verbose, "...retrieving sequence...")
-  queryseq <- list(db = "sequences", id = gisuse[,1], rettype = "fasta", retmode = "text")
-  outseq <- content(GET("https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi", query = queryseq), "text", encoding = "UTF-8")
+  queryseq <- list(db = "sequences", id = gisuse[,1], rettype = "fasta",
+    retmode = "text", api_key = ncbi_key())
+  con <- crul::HttpClient$new("https://eutils.ncbi.nlm.nih.gov")
+  w <- con$get("entrez/eutils/efetch.fcgi", query = traitsc(queryseq))
+  w$raise_for_status()
+  outseq <- w$parse("UTF-8")
   seq <- gsub("\n", "", strsplit(sub("\n", "<<<", outseq), "<<<")[[1]][[2]])
   accessnum <- strsplit(outseq, "\\|")[[1]][4]
   outt <- list(xx, as.character(gisuse[,3]), gisuse[,1], accessnum, gisuse[,2], seq)
