@@ -19,11 +19,18 @@
 #' accession number prefixs (XM and XR). This can result in less than the
 #' \code{limit} being returned even if there are more sequences available,
 #' since this filtering is done after searching NCBI.
+#' @param sleep (integer) number of seconds to sleep before each HTTP request.
+#' use if running to 429 Too Many Requests errors from NCBI. default: 0 
+#' (no sleep)
 #' @return \code{data.frame} of results if a single input is given. A list of
 #'  \code{data.frame}s if multiple inputs are given.
 #' @seealso \code{\link{ncbi_byid}}, \code{\link{ncbi_byname}}
-#' @author Scott Chamberlain \email{myrmecocystus@@gmail.com}, Zachary Foster
-#'   \email{zacharyfoster1989@@gmail.com}
+#' @author Scott Chamberlain, Zachary Foster \email{zacharyfoster1989@@gmail.com}
+#' @section Authentication:
+#' NCBI rate limits requests. If you set an API key you have a higher rate limit. 
+#' Set your API key like `Sys.setenv(ENTREZ_KEY="yourkey")` or you can use 
+#' `?rentrez::set_entrez_key`. set verbose curl output (`crul::set_verbose()`) to
+#' make sure your api key is being sent in the requests
 #' @examples \dontrun{
 #' # A single species
 #' out <- ncbi_searcher(taxa="Umbra limi", seqrange = "1:2000")
@@ -65,7 +72,9 @@
 #' }
 ncbi_searcher <- function(taxa = NULL, id = NULL, seqrange="1:3000",
   getrelated=FALSE, fuzzy=FALSE, limit = 500, entrez_query = NULL,
-  hypothetical = FALSE, verbose=TRUE) {
+  hypothetical = FALSE, verbose=TRUE, sleep=0L) {
+
+  cat(paste0("using sleep: ", sleep), sep="\n")
 
   # Argument validation ----------------------------------------------------------------------------
   if (sum(c(is.null(taxa), is.null(id))) != 1) {
@@ -86,12 +95,12 @@ ncbi_searcher <- function(taxa = NULL, id = NULL, seqrange="1:3000",
   # look up sequences for taxa ids -----------------------------------------------------------------
   if (length(id) == 1) {
     ncbi_searcher_foo(id, getrelated = getrelated, verbose = verbose,
-                      seqrange = seqrange, entrez_query = entrez_query, fuzzy = fuzzy,
-                      limit = limit, hypothetical = hypothetical)
+      seqrange = seqrange, entrez_query = entrez_query, fuzzy = fuzzy,
+      limit = limit, hypothetical = hypothetical, sleep=sleep)
   } else {
     lapply(id, ncbi_searcher_foo, getrelated = getrelated, verbose = verbose,
-           seqrange = seqrange, entrez_query = entrez_query, fuzzy = fuzzy, limit = limit,
-           hypothetical = hypothetical)
+      seqrange = seqrange, entrez_query = entrez_query, fuzzy = fuzzy, limit = limit,
+      hypothetical = hypothetical, sleep=sleep)
   }
 }
 
@@ -100,20 +109,23 @@ url_esearch <- "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi"
 url_esummary <- "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi"
 
 # Function to process queries one at a time ------------------------------------------------------
-ncbi_searcher_foo <- function(xx, getrelated, verbose, seqrange, entrez_query, fuzzy, limit, hypothetical, ...) {
+ncbi_searcher_foo <- function(xx, getrelated, verbose, seqrange, entrez_query,
+  fuzzy, limit, hypothetical, sleep, ...) {
+
   # Search for sequence IDs for the given taxon  - - - - - - - - - - - - - - - - - - - - - - - - -
   mssg(verbose, paste("Working on ", names(xx), "...", sep = ""))
   mssg(verbose, "...retrieving sequence IDs...")
-  seq_ids <- search_for_sequences(xx, seqrange, entrez_query, fuzzy, limit, ...)
+  seq_ids <- search_for_sequences(xx, seqrange, entrez_query, fuzzy, limit, sleep, ...)
   # Search for sequences of the taxons parent if necessary and possible  - - - - - - - - - - - - -
   if (is.null(seq_ids) && getrelated) {
     mssg(verbose, paste("no sequences for ", names(xx), " - getting other related taxa", sep = ""))
-    parent_id <- get_parent(xx, verbose)
+    parent_id <- get_parent(xx, verbose, sleep)
     if (is.na(parent_id)) {
       mssg(verbose, paste0("no related taxa found"))
     } else {
       mssg(verbose, paste0("...retrieving sequence IDs for ", names(xx), "..."))
-      seq_ids <- search_for_sequences(parent_id, seqrange, entrez_query, fuzzy, limit, ...)
+      seq_ids <- search_for_sequences(parent_id, seqrange, entrez_query, fuzzy,
+        limit, sleep, ...)
     }
   }
   # Retrieve sequence information  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -122,7 +134,7 @@ ncbi_searcher_foo <- function(xx, getrelated, verbose, seqrange, entrez_query, f
     df <- data.frame(character(0), numeric(0), character(0), character(0), numeric(0), stringsAsFactors = FALSE)
   } else {
     mssg(verbose, "...retrieving available genes and their lengths...")
-    df <- download_summary(seq_ids, hypothetical = hypothetical, ...)
+    df <- download_summary(seq_ids, hypothetical=hypothetical, sleep=sleep, ...)
     mssg(verbose, "...done.")
   }
   # Format output  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -130,7 +142,7 @@ ncbi_searcher_foo <- function(xx, getrelated, verbose, seqrange, entrez_query, f
 }
 
 # Function to search for sequences with esearch --------------------------------------------------
-search_for_sequences <- function(id, seqrange, entrez_query, fuzzy, limit, ...) {
+search_for_sequences <- function(id, seqrange, entrez_query, fuzzy, limit, sleep=0L, ...) {
   if (is.na(id)) return(NULL)
   # Construct search query  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   ## fuzzy or not fuzzy
@@ -140,12 +152,14 @@ search_for_sequences <- function(id, seqrange, entrez_query, fuzzy, limit, ...)
     sprintf("txid%s AND %s[SLEN]", id, seqrange)
   }
   if (!is.null(entrez_query)) query_term <- paste(query_term, entrez_query, sep = " AND ")
-  query <- list(db = "nuccore", retmax = limit, term = query_term)
+  query <- list(db = "nuccore", retmax = limit, term = query_term, api_key = ncbi_key())
   # Submit query to NCBI - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  query_init <- GET(url_esearch, query = query, ...)
-  stop_for_status(query_init)
+  con <- crul::HttpClient$new(url_esearch, opts = list(...))
+  Sys.sleep(sleep)
+  res <- con$get(query = traitsc(query))
+  res$raise_for_status()
   # Parse result - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-  query_content <- xml2::read_xml(content(query_init, "text", encoding = "UTF-8"))
+  query_content <- xml2::read_xml(res$parse("UTF-8"))
   esearch_result <- xml2::xml_find_all(query_content, "//eSearchResult")[[1]]
   if (as.numeric(xml2::xml_text(xml2::xml_find_all(esearch_result, "//Count")[[1]])) == 0) {
     return(NULL)
@@ -155,7 +169,7 @@ search_for_sequences <- function(id, seqrange, entrez_query, fuzzy, limit, ...)
 }
 
 # Function to download and parse sequence summary information using esummary ---------------------
-download_summary <- function(seq_id, hypothetical, ...) {
+download_summary <- function(seq_id, hypothetical, sleep=0L, ...) {
   actualnum <- length(seq_id)
   if (actualnum > 10000) {
     q <- list(db = "nucleotide")
@@ -166,66 +180,69 @@ download_summary <- function(seq_id, hypothetical, ...) {
       q$id = paste(seq_id[getstart[i]:(getstart[i] + (getnum[i] - 1))], collapse = " ")
       q$retstart <- getstart[i]
       q$retmax <- getnum[i]
-      query_res <- POST(url_esummary, body = q)
-      stop_for_status(query_res)
-      iterlist[[i]] <- parseres(query_res, hypothetical)
+      con <- crul::HttpClient$new(url_esummary, opts = list(...))
+      Sys.sleep(sleep)
+      res <- con$post(body = traitsc(q))
+      res$raise_for_status()
+      iterlist[[i]] <- parseres(res, hypothetical, sleep)
     }
     data.frame(rbindlist(iterlist), stringsAsFactors = FALSE)
   } else {
-    q <- list(db = "nucleotide", id = paste(seq_id, collapse = " "))
-    query_res <- POST(url_esummary, body = q, ...)
-    stop_for_status(query_res)
-    parseres(query_res, hypothetical)
+    body <- list(db = "nucleotide", api_key = ncbi_key(), id = paste(seq_id, collapse = " "))
+    con <- crul::HttpClient$new(url_esummary, opts = list(...))
+    Sys.sleep(sleep)
+    res <- con$post(body = traitsc(body))
+    res$raise_for_status()
+    parseres(res, hypothetical, sleep)
   }
 }
 
 # Function to get a taxon's parent ---------------------------------------------------------------
-get_parent <- function(id, verbose) {
+get_parent <- function(id, verbose, sleep=0L) {
   if (!is.na(id)) {
+    Sys.sleep(sleep)
     ancestry <- classification(id = id, db = "ncbi")[[1]]
     if (nrow(ancestry) > 1) {
       parent_name <- ancestry$name[nrow(ancestry) - 1]
+      Sys.sleep(sleep)
       return(get_uid(parent_name, messages = verbose))
     }
   }
   if (!is.null(names(id)) && grepl(" ", names(id))) { #if a name is given and looks like a species
     parent_name <- strsplit(names(id), " ")[[1]][[1]]
+    Sys.sleep(sleep)
     return(get_uid(parent_name, messages = verbose))
   }
   return(NA)
 }
 
 # Function to parse results from http query ------------------------------------------------------
-parseres <- function(x, hypothetical){
-  outsum <- xml2::xml_find_all(xml2::read_xml(content(x, "text", encoding = "UTF-8")), "//eSummaryResult")[[1]]
+parseres <- function(x, hypothetical, sleep){
+  outsum <- xml2::xml_find_all(xml2::read_xml(x$parse("UTF-8")), "//eSummaryResult")[[1]]
   names <- xml2::xml_attr(xml2::xml_find_all(outsum, "//Item"), attr = "Name") # gets names of values in summary
   predicted <- as.character(xml2::xml_text(xml2::xml_find_all(outsum, "//Item"))[grepl("Caption", names)]) #  get access numbers
   has_access_prefix <- grepl("_", predicted)
   access_prefix <- unlist(Map(function(x, y) ifelse(x, strsplit(y, "_")[[1]][[1]], NA),
                               has_access_prefix, predicted))
-#   predicted[has_access_prefix] <- vapply(strsplit(predicted[has_access_prefix], "_"), `[[`, character(1), 2)
-#
   length_ <- as.numeric(xml2::xml_text(xml2::xml_find_all(outsum, "//Item"))[grepl("Length", names)]) # gets seq lengths
   gis <- as.numeric(xml2::xml_text(xml2::xml_find_all(outsum, "//Item"))[grepl("Gi", names)]) # gets GI numbers
-
-  spused <- taxonomy(zz = outsum)
+  spused <- taxonomy(outsum, sleep)
   desc <- xml2::xml_text(xml2::xml_find_all(outsum, "//Item"))[grepl("Title", names)] # gets seq lengths # get spp names
-  #   spused <- sapply(spnames, function(x) paste(strsplit(x, " ")[[1]][1:2], sep = "", collapse = " "))
-
-  # genesavail <- sapply(spnames, function(x) paste(strsplit(x, " ")[[1]][-c(1:2)], sep = "", collapse = " "), USE.NAMES = FALSE)
   df <- data.frame(spused = spused, length = length_, genesavail = desc, access_num = predicted, ids = gis, stringsAsFactors = FALSE)
   if (!hypothetical) df <- df[!(access_prefix %in% c("XM","XR")), ]
   return(df)
 }
 
-taxonomy <- function(zz) {
+taxonomy <- function(zz, sleep=0L) {
   taxids <- xml2::xml_text(xml2::xml_find_all(zz, '//Item[@Name="TaxId"]'))
   uids <- unique(taxids)
   out <- list()
   for (i in seq_along(uids)) {
-    res <- GET(paste0(url_esummary, "?db=taxonomy&id=", uids[i]))
-    stop_for_status(res)
-    xml <- xml2::read_xml(content(res, "text", encoding = "UTF-8"))
+    con <- crul::HttpClient$new(url_esummary)
+    Sys.sleep(sleep)
+    res <- con$get(query = traitsc(list(db="taxonomy", id=uids[i], api_key=ncbi_key())))
+    res$raise_for_status()
+    xml <- xml2::read_xml(res$parse("UTF-8"))
     out[[ uids[i] ]] <- xml2::xml_text(xml2::xml_find_all(xml, '//Item[@Name="ScientificName"]'))
   }
   for (i in seq_along(out)) {
@@ -237,3 +254,5 @@ taxonomy <- function(zz) {
   }
   return(taxids)
 }
+
+ncbi_key <- function() Sys.getenv("ENTREZ_KEY", "") %||% NULL
